enyo.kind({
	name: "JobStatus",
	kind: enyo.Item,
	tapHighlight: false,
	className: "JobStatusItem enyo-item",
	jobCancelled: false,
	statusBeforeCanceling: "",
	published: {
		jobID: "",
		printerName: "",
		description: "",
		currentStatus: "",
		pageNumber: -1,
		totalPages: -1,
		blockedReasons: []
	},
	events: {
		onCancellableChange: ""
	},
	components: [
		{kind: "PrintManagerService", components: [
			{name: "cancelJob", method: "jobs/cancel", onSuccess: "cancelJobSuccess", onFailure: "cancelJobFailure"},
			{name: "cancelJobFromQueue", method: "jobs/cancel", onSuccess: "cancelJobFromQueueSuccess", onFailure: "cancelJobFromQueueFailure"}
		]},
		{kind: "HFlexBox", className: "job", align: "center", pack: "justify", components: [
			{content: "", name: "currentStatusLabel", className: "status"},
			{kind: "VFlexBox", className: "JobStatusInfo", flex: 1, components: [
				{content: "", name: "descriptionLabel", className: "name"},
				{content: "", name: "printerNameLabel", className: "printer"},
			]},
			{kind: "IconButton", name: "cancel", className: "cancel enyo-button-light", label: "", icon: "images/btn_job_cancel.png", onclick: "cancelClick"}
		]},
		{kind: "MessageBox", name:"messageBox"}
	],
	create: function() {
		this.pageNumber = -1;
		this.totalPages = -1;
		this.inherited(arguments);
		this.printerNameChanged();
		this.descriptionChanged();
		this.currentStatusChanged();
		this.pageNumberChanged();
		this.totalPagesChanged();
		this.blockedReasonsChanged();
	},
	printerNameChanged: function() {
		this.$.printerNameLabel.setContent(this.getPrinterName());
	},
	descriptionChanged: function() {
		this.updateDescriptionLabel();
	},
	currentStatusChanged: function() {
		var statusText = "";

		if (this.getCurrentStatus() === "DONE" || this.getCurrentStatus() === "Complete") {
			statusText = $L("Complete");
			this.$.currentStatusLabel.setClassName("status ok");
			this.$.cancel.setDisabled(true);
			this.doCancellableChange();
		}
		else if (this.getCurrentStatus() === "RUNNING") {
			statusText = $L("Printing...");
			this.$.currentStatusLabel.setClassName("status");
		}
		else if (this.getCurrentStatus() === "QUEUED") {
			statusText = $L("Waiting");
			this.$.currentStatusLabel.setClassName("status");
		}
		else if (this.getCurrentStatus() === "Canceled") {
			statusText = $L("Canceled");
			this.$.currentStatusLabel.setClassName("status error");
			this.$.cancel.setDisabled(true);
			this.doCancellableChange();
		}
		else if (this.getCurrentStatus() === "Canceling") {
			statusText = $L("Canceling...");
			this.$.currentStatusLabel.setClassName("status error");
			this.$.cancel.setDisabled(true);
			this.doCancellableChange();
		}
		else if (this.getCurrentStatus() === "BLOCKED") {
			if (this.getErrorText() !== "") {	// we're blocked by an error state
				statusText = $L("Blocked");
				this.$.currentStatusLabel.setClassName("status error");
			}
			else {			// we're blocked but its only a warning state, so display state as printing
				statusText = $L("Printing...");
				this.$.currentStatusLabel.setClassName("status");
			}
		}
		else if (this.getCurrentStatus() === "Failed") {
			statusText = $L("Failed");
			this.$.currentStatusLabel.setClassName("status error");
			this.$.cancel.setDisabled(true);
			this.doCancellableChange();
		}
		else if (this.getCurrentStatus() === "Preparing") {
			statusText = $L("Preparing...");
			this.$.currentStatusLabel.setClassName("status");
		}

		this.$.currentStatusLabel.setContent(statusText);
		this.updateDescriptionLabel();
	},
	pageNumberChanged: function() {
		console.log("JobStatus pageNumberChanged: pageNumber = " + this.getPageNumber());
		if (this.getPageNumber() === "") {
			this.setPageNumber(-1);
		}
		this.updateDescriptionLabel();
	},
	totalPagesChanged: function() {
		console.log("JobStatus totalPagesChanged: totalPages = " + this.getTotalPages());
		if (this.getTotalPages() === "") {
			this.setTotalPages(-1);
		}
		this.updateDescriptionLabel();
	},
	blockedReasonsChanged: function() {
		console.log("JobStatus blockedReasonsChanged: blockedReasons = " + JSON.stringify(this.getBlockedReasons()));
		this.updateDescriptionLabel();
	},
	cancelClick: function() {
		if (this.jobCancelled === false) {
			this.$.cancel.setDisabled(true);
			this.doCancellableChange();
			this.statusBeforeCanceling = this.getCurrentStatus();
			this.setCurrentStatus("Canceling");
			this.$.cancelJob.call({jobID: this.jobID});
		}
	},
	cancelJobSuccess: function(inSender, inResponse, inRequest) {
		console.log("JobStatus cancelJobSuccess: " + JSON.stringify(inResponse));
		this.jobCancelled = true;
		this.setCurrentStatus("Canceled");
	},
	cancelJobFailure: function(inSender, inResponse, inRequest) {
		console.log("JobStatus cancelJobFailure: " + JSON.stringify(inResponse));
		console.log(inSender.getName() + " FAIL: " + inResponse.errorText);
		this.$.cancel.setDisabled(false);
		this.doCancellableChange();
		this.jobCancelled = false;
		this.setCurrentStatus(this.statusBeforeCanceling);
		this.$.messageBox.showMessage(PrintManagerError.getErrorText(inResponse.errorCode));
	},
	updateDescriptionLabel: function() {
		var errorString = this.getErrorText();
		console.log("JobStatus updateDescriptionLabel: errorString = " + errorString);

		if (this.getCurrentStatus() === "BLOCKED" && errorString !== "") {
			this.$.descriptionLabel.setContent(errorString);
		}
		else {
			if (this.getPageNumber() !== -1 && this.getPageNumber() !== "") {
				console.log("JobStatus updateDescriptionLabel: pageNumber = " + this.getPageNumber());
				var descLabel = "", descTemplate;
				if (this.getTotalPages() !== -1 && this.getTotalPages() !== 0 && this.getTotalPages() !== "") {
					console.log("JobStatus updateDescriptionLabel: totalPages = " + this.getTotalPages());
					descTemplate = new enyo.g11n.Template($L("Page #{pageNumber} of #{totalPages} from #{originName}"));
					descLabel = descTemplate.evaluate({pageNumber: this.getPageNumber(),
					                                   totalPages: this.getTotalPages(),
					                                   originName: this.getDescription()
						});
				}
				else {
					descTemplate = new enyo.g11n.Template($L("Page #{pageNumber} from #{originName}"));
					descLabel = descTemplate.evaluate({pageNumber: this.getPageNumber(),
					                                   originName: this.getDescription()
						});
				}

				this.$.descriptionLabel.setContent(descLabel);
			}
		}
	},
	cancelJobFromQueue: function() {
		console.log("JobStatus cancelJobFromQueue");
		if (this.jobCancelled === false) {
			this.$.cancel.setDisabled(true);
			this.doCancellableChange();
			this.statusBeforeCanceling = this.getCurrentStatus();
			this.setCurrentStatus("Canceling");
			this.$.cancelJobFromQueue.call({jobID: this.jobID});
		}
	},
	cancelJobFromQueueSuccess: function(inSender, inResponse, inRequest) {
		console.log("JobStatus cancelJobFromQueueSuccess: " + JSON.stringify(inResponse));
		this.jobCancelled = true;
		this.setCurrentStatus("Canceled");
	},
	cancelJobFromQueueFailure: function(inSender, inResponse, inRequest) {
		console.log("JobStatus cancelJobFromQueueFailure: " + JSON.stringify(inResponse));
		console.log(inSender.getName() + " FAIL: " + inResponse.errorText);
		this.$.cancel.setDisabled(false);
		this.doCancellableChange();
		this.jobCancelled = false;
		this.setCurrentStatus(this.statusBeforeCanceling);
	},
	getErrorText: function() {
		var errorFlags = [];
		errorFlags["unableToConnect"] = false;
		errorFlags["busy"] = false;
		errorFlags["cancelled"] = false;
		errorFlags["outOfPaper"] = false;
		errorFlags["outOfInk"] = false;
		errorFlags["outOfToner"] = false;
		errorFlags["jammed"] = false;
		errorFlags["doorOpen"] = false;
		errorFlags["serviceRequest"] = false;
		errorFlags["lowOnInk"] = false;
		errorFlags["lowOnToner"] = false;

		// find all errors currently present
		for (var i = 0; i < this.blockedReasons.length; i++) {
			if (this.blockedReasons[i] === "Unable_To_Connect") {
				errorFlags["unableToConnect"] = true;
			}
			else if (this.blockedReasons[i] === "Busy") {
				errorFlags["busy"] = true;
			}
			else if (this.blockedReasons[i] === "Cancelled") {
				errorFlags["cancelled"] = true;
			}
			else if (this.blockedReasons[i] === "Out_Of_Paper") {
				errorFlags["outOfPaper"] = true;
			}
			else if (this.blockedReasons[i] === "Out_Of_Ink") {
				errorFlags["outOfInk"] = true;
			}
			else if (this.blockedReasons[i] === "Out_Of_Toner") {
				errorFlags["outOfToner"] = true;
			}
			else if (this.blockedReasons[i] === "Jammed") {
				errorFlags["jammed"] = true;
			}
			else if (this.blockedReasons[i] === "Door_Open") {
				errorFlags["doorOpen"] = true;
			}
			else if (this.blockedReasons[i] === "Service_Request") {
				errorFlags["serviceRequest"] = true;
			}
			else if (this.blockedReasons[i] === "Low_On_Ink") {
				errorFlags["lowOnInk"] = true;
			}
			else if (this.blockedReasons[i] === "Low_On_Toner") {
				errorFlags["lowOnToner"] = true;
			}
		}

		// look through error flags in priority order to display most important error message
		if (errorFlags["doorOpen"] === true) {
			return $L("Door Open");
		}
		else if (errorFlags["outOfPaper"] === true) {
			return $L("Out of Paper");
		}
		else if (errorFlags["jammed"] === true) {
			return $L("Printer Jam");
		}
		else if (errorFlags["serviceRequest"] === true) {
			return $L("Printer Error");
		}
		else if (errorFlags["outOfInk"] === true) {
			return $L("Out of Ink");
		}
		else if (errorFlags["outOfToner"] === true) {
			return $L("Out of Toner");
		}
		else if (errorFlags["unableToConnect"] === true) {
			return $L("Unable To Connect");
		}
		else if (errorFlags["cancelled"] === true) {
			return $L("Print Job Canceled");
		}

		// These are warning messages only and aren't displayed in the print queue
		else if (errorFlags["busy"] === true) {
			return "";
		}
		else if (errorFlags["lowOnInk"] === true) {
			return "";
		}
		else if (errorFlags["lowOnToner"] === true) {
			return "";
		}

		// use general service request message as fallback if reasons list is empty or no valid reason found
		else {
			return $L("Printer Error");
		}

		return "";
	},
	changeRenderStatus: function(newStatus) {
		console.log("JobStatus changeRenderStatus: newStatus = " + newStatus);
		console.log("JobStatus changeRenderStatus: currentStatus = " + this.getCurrentStatus());

		if (newStatus === "Preparing" && this.getCurrentStatus() === "QUEUED") {
			console.log("JobStatus changeRenderStatus: changing status to Preparing");
			this.setCurrentStatus("Preparing");
		}
		else if (newStatus === "PreparingDone" && this.getCurrentStatus() === "Preparing") {
			console.log("JobStatus changeRenderStatus: changing status to QUEUED");
			this.setCurrentStatus("QUEUED");
		}
		else if (newStatus === "QUEUED" && this.getCurrentStatus() !== "Preparing") {
			console.log("JobStatus changeRenderStatus: changing status to QUEUED");
			this.setCurrentStatus("QUEUED");
		}
	},
	isCancellable: function() {
		var cancelButtonDisabled = this.$.cancel.getDisabled();
		console.log("JobStatus isCancellable: cancel disabled = " + cancelButtonDisabled);
		if (cancelButtonDisabled === true) {
			return false;
		}
		else {
			return true;
		}
	}
});
