enyo.kind({
	name: "enyo.Canon.PrintMonitor",
	kind: enyo.Control,
	idsStarted: [],
	dashboardCount: 0,
	components: [
		{kind: "PrintManagerService", onFailure: "printManagerFailure", components: [
			{name: "getRenderStatus", method: "jobs/getRenderStatus", subscribe: true, onSuccess: "renderStatusSuccess"},
			{name: "getStatus", method: "jobs/getStatus", subscribe: true, onSuccess: "jobStatusSuccess"}
		]},
		{kind: "PrintMonitorQueueStatus", name: "queueStatus", onListUpdateNeeded: "queueListUpdate"}
	],
	create: function() {
		this.inherited(arguments);
		console.log("PrintMonitor started");
		this.$.getRenderStatus.call();
		this.$.getStatus.call();
	},
	renderStatusSuccess: function(inSender, inResponse, inRequest) {
		console.log("PrintMonitor renderStatusSuccess: inResponse = " + JSON.stringify(inResponse));
		var res = inResponse;
		console.log("PrintMonitor renderStatusSuccess: res = " + JSON.stringify(res));
		if (typeof res.jobID !== "undefined") {
			this.updateDashboardRender(res);
		}
	},
	jobStatusSuccess: function(inSender, inResponse, inRequest) {
		console.log("PrintMonitor jobStatusSuccess: inResponse = " + JSON.stringify(inResponse));
		var res = inResponse;
		console.log("PrintMonitor jobStatusSuccess: res = " + JSON.stringify(res));
		if (typeof res.jobID !== "undefined") {
			this.updateStatus(res);
		}
	},
	updateStatus: function(result) {
		var jobID = result.jobID;
		console.log("PrintMonitor updateStatus: jobID = " + jobID);
		this.updateDashboard(result, jobID);
		this.updatePopup(result, jobID);
	},
	updateDashboardRender: function(result) {
		var jobID = result.jobID;
		console.log("PrintMonitor updateDashboardRender: jobID = " + jobID);

		if (typeof result !== "undefined" && typeof result.renderResultCode !== "undefined" &&
		    result.renderResultCode < 0) {
			console.log("PrintMonitor updateDashboardRender: renderResultCode = " + result.renderResultCode);
			console.log("PrintMonitor updateDashboardRender: skipped processing due to error code received");
			return;
		}

		if (this.idsStarted.indexOf(jobID) === -1) {			// haven't displayed preparing message yet
			var idsStartedLength = this.idsStarted.length;
			this.idsStarted[idsStartedLength] = jobID;

			console.log("PrintMonitor updateDashboardRender: Displaying preparing banner message");
			enyo.windows.addBannerMessage($L("Preparing Print Job..."), "{}", "images/notif24/normal.png");
		}

		this.$.queueStatus.updateJobItem(jobID, 'Preparing', -1, -1, "", []);
		this.displayDashboardChanges();
	},
	updateDashboard: function(result, jobID) {
		var printerState = "";
		var currentPage = -1;
		var totalPages = -1;
		var description = "";
		var blockedReasons = [];
		if (typeof result.printerState !== "undefined") {
			printerState = result.printerState;
		}
		if (typeof result.currentPage !== "undefined") {
			currentPage = result.currentPage;
		}
		if (typeof result.totalPages !== "undefined") {
			totalPages = result.totalPages;
		}
		if (typeof result.appName !== "undefined") {
			description = result.appName;
		}

		if (typeof result.blockedReasons !== "undefined") {
			blockedReasons = result.blockedReasons;

			if (printerState === "RUNNING" &&
			    (result.blockedReasons.indexOf("Low_On_Ink") !== -1 ||
			     result.blockedReasons.indexOf("Low_On_Toner") !== -1)) {
				console.log("PrintMonitor updateDashboard: overriding printer state to BLOCKED");
				printerState = "BLOCKED";
			}
		}

		if (result.statusId === "JOBSTATE" && result.printerState === "DONE" &&
		    (result.jobStatus === "Cancelled" || result.jobStatus === "Error" || result.jobStatus === "Corrupt")) {
			if (result.jobStatus === "Cancelled") {
				printerState = "Cancelled";
				blockedReasons = ["Cancelled"];
			}
			else if (result.jobStatus === "Error" || result.jobStatus === "Corrupt") {
				printerState = "Failed";
				blockedReasons = ["Failed"];
			}
			console.log("PrintMonitor updateDashboard: blockedReasons = " + JSON.stringify(blockedReasons));
		}

		this.$.queueStatus.updateJobItem(jobID, printerState, currentPage, totalPages, description, blockedReasons);
		this.displayDashboardChanges();
	},
	displayDashboardChanges: function() {
		var dashboardItem = this.findDashboard();
		console.log("PrintMonitor displayDashboardChanges: dashboardItem = " + dashboardItem);

		var dashboardLayers = this.$.queueStatus.getLayers();
		console.log("PrintMonitor displayDashboardChanges: dashboardLayers = " + JSON.stringify(dashboardLayers));
		if (typeof dashboardLayers === "undefined") {
			return;
		}

		var dashboardLayersLength = dashboardLayers.length;
		var dashboardTopLayerIcon = "";
		if (dashboardLayersLength > 0) {
			var dashboardLastItem = dashboardLayers[dashboardLayersLength - 1];
			dashboardTopLayerIcon = dashboardLastItem.icon;
			dashboardTopLayerIcon = dashboardTopLayerIcon.replace("notif48", "notif24");
		}
		console.log("PrintMonitor displayDashboardChanges: dashboardTopLayerIcon = " + dashboardTopLayerIcon);

		var dashboardName;
		if (typeof dashboardItem === "undefined" && dashboardLayers.length > 0) {	// need to create the dashboard
			this.dashboardCount++;
			dashboardName = "printDashboard" + this.dashboardCount;
			console.log("PrintMonitor displayDashboardChanges: dashboardName = " + dashboardName);

			this.createComponent({kind: "Dashboard",
			                      name: dashboardName,
			                      smallIcon: dashboardTopLayerIcon,
			                      onTap: "dashboardTapped",
			                      onLayerSwipe: "dashboardSwiped",
			                      onUserClose: "dashboardSwiped"
				});

			this.$[dashboardName].setLayers(dashboardLayers);
		}
		else if (typeof dashboardItem !== "undefined" && dashboardLayers.length === 0) {	// need to remove the dashboard
			dashboardItem.destroy();
		}
		else if (typeof dashboardItem !== "undefined" && dashboardLayers.length > 0) {	// need to update dashboard
			var currentDashboardIcon = dashboardItem.getSmallIcon();
			console.log("PrintMonitor displayDashboardChanges: currentDashboardIcon = " + currentDashboardIcon);

			if (currentDashboardIcon !== dashboardTopLayerIcon) {
				this.dashboardCount++;
				dashboardName = "printDashboard" + this.dashboardCount;
				console.log("PrintMonitor displayDashboardChanges: dashboardName = " + dashboardName);

				dashboardItem.destroy();
				this.createComponent({kind: "Dashboard",
				                      name: dashboardName,
				                      smallIcon: dashboardTopLayerIcon,
				                      onTap: "dashboardTapped",
				                      onLayerSwipe: "dashboardSwiped",
				                      onUserClose: "dashboardSwiped"
					});
			}
			else {
				dashboardName = "printDashboard" + this.dashboardCount;
				console.log("PrintMonitor displayDashboardChanges: dashboardName = " + dashboardName);
			}

			this.$[dashboardName].setLayers(dashboardLayers);
		}
	},
	dashboardTapped: function() {
		enyo.windows.activate("queue.html", "printQueue", "", {});
	},
	dashboardSwiped: function(inSender) {
		if (typeof inSender === "undefined" || typeof inSender.name === "undefined") {
			return;
		}

		var senderName = inSender.name;
		console.log("PrintMonitor dashboardSwiped: senderName = " + senderName);
		var currentDashboardName = "printDashboard" + this.dashboardCount;
		console.log("PrintMonitor dashboardSwiped: currentDashboardName = " + currentDashboardName);

		if (senderName !== currentDashboardName) {
			console.log("PrintMonitor dashboardSwiped: event from old dashboard, aborting");
			return;
		}

		this.$.queueStatus.dashboardSwiped();
		this.displayDashboardChanges();
	},
	queueListUpdate: function() {
		this.displayDashboardChanges();
	},
	findDashboard: function() {
		var monitorItems = this.getComponents();
		for (var i = 0; i < monitorItems.length; i++) {
			if (monitorItems[i].kind === "Dashboard") {
				return monitorItems[i];
			}
		}
		return undefined;
	},
	updatePopup: function(result, jobID) {
		var popupStatus = enyo.windows.fetchWindow("printPopup" + jobID);
		console.log("PrintMonitor updatePopup: popupStatus = " + popupStatus);

		var blockingAlertExists = false;
		if (typeof result.blockedReasons !== "undefined") {
			blockingAlertExists = this.findBlockingAlert(result.blockedReasons);
		}

		var blockedReasons;
		var params;
		if (result.statusId === "JOBSTATE" && result.printerState === "BLOCKED" && blockingAlertExists) {
			blockedReasons = [];
			if (typeof result.blockedReasons !== "undefined") {
				blockedReasons = result.blockedReasons;
			}

			if (typeof popupStatus === "undefined") {	// popup not currently showing
				console.log("PrintMonitor updatePopup: blockedReasons = " + JSON.stringify(blockedReasons));
				params = {jobID: jobID,
				          printerState: result.printerState,
				          blockedReasons: blockedReasons
				         };
				console.log("PrintMonitor updatePopup: params = " + JSON.stringify(params));
				enyo.windows.openPopup("popup.html",
			 	                       "printPopup" + jobID,
			 	                       params,
				                       "",
			 	                       120,
				                       false);
			}
			else {			// popup already showing - update status displayed
				console.log("PrintMonitor updatePopup: blockedReasons = " + JSON.stringify(blockedReasons));
				params = {printerState: result.printerState,
				          blockedReasons: blockedReasons
				         };
				console.log("PrintMonitor updatePopup: params = " + JSON.stringify(params));
				enyo.windows.setWindowParams(popupStatus, params);
			}
		}
		else if (typeof popupStatus !== "undefined" && typeof result.printerState !== "undefined") {
			// send other status update so we remove popup
			params = {printerState: result.printerState};
			console.log("PrintMonitor updatePopup: params = " + JSON.stringify(params));
			enyo.windows.setWindowParams(popupStatus, params);
		}
	},
	findBlockingAlert: function(blockedReasons) {
		if (typeof blockedReasons === "undefined") {
			return false;
		}

		for (var i = 0; i < blockedReasons.length; i++) {
			if (blockedReasons[i] !== "" &&
			    blockedReasons[i] !== "Cancelled" &&		// these blocked reasons are only warnings, so don't flag as blocking alert
			    blockedReasons[i] !== "Busy" &&
			    blockedReasons[i] !== "Low_On_Ink" &&
			    blockedReasons[i] !== "Low_On_Toner") {
				return true;
			}
		}
		return false;
	},
	printManagerFailure: function(inSender, inResponse) {
		console.log("PrintMonitor printManagerFailure: " + JSON.stringify(inResponse));
		console.log(inSender.getName() + " FAIL: " + inResponse.errorText);
	}
});
