enyo.kind({
	name: "PrintMonitorQueueStatus",
	kind: enyo.Control,
	jobItems: [],
	idsShown: [],
	events: {
		onListUpdateNeeded: ""
	},
	components: [],
	create: function() {
		this.inherited(arguments);
	},
	updateJobItem: function(jobID, printerState, currentPage, totalPages, description, blockedReasons) {
		var itemUpdated = false;

		for (var i = 0; (itemUpdated === false) && (i < this.jobItems.length); i++) {
			if (this.jobItems[i].jobID == jobID) {
				this.updateExistingJobItem(i, printerState, currentPage, totalPages, description, blockedReasons);
				itemUpdated = true;
			}
		}

		if (itemUpdated === false) {
			this.addNewJobItem(jobID, printerState, currentPage, totalPages, description, blockedReasons);
		}

		if (printerState === "Cancelled" || printerState === "Failed") {
			console.log("PrintMonitorQueueStatus updateJobItem: printerState = " + printerState);
			console.log("PrintMonitorQueueStatus updateJobItem: setting timer to remove dashboard");
			setTimeout(enyo.bind(this, "delayedRemoveJobItem", jobID), 6000);
		}
	},
	delayedRemoveJobItem: function(jobID) {
		console.log("PrintMonitorQueueStatus delayedRemoveJobItem: jobID = " + jobID);
		this.updateJobItem(jobID, "DONE", -1, -1, "", []);
		this.doListUpdateNeeded();
	},
	updateExistingJobItem: function(i, printerState, currentPage, totalPages, description, blockedReasons) {
		console.log("PrintMonitorQueueStatus updateExistingJobItem: printerState = " + printerState);
		if (printerState !== "") {
			if (printerState === "DONE") {
				console.log("PrintMonitorQueueStatus updateExistingJobItem: removing item " + i);
				this.removeJobItem(i);
				return;
			}
			else if ((this.jobItems[i].printerState !== "RUNNING"   && this.jobItems[i].printerState !== "BLOCKED" &&
			          this.jobItems[i].printerState !== "Cancelled" && this.jobItems[i].printerState !== "Failed") || 
			         printerState !== "Preparing") {
				if (printerState === "Preparing" || printerState === "RUNNING" || printerState === "BLOCKED" ||
				    printerState === "Cancelled" || printerState === "Failed") {
					// limit stored printer state to Preparing, Running, Blocked, Cancelled, or Failed
					// we'll still store other details about job, but won't update state to other values
					this.jobItems[i].printerState = printerState;
				}
			}
		}
		if (currentPage !== "" && currentPage !== -1) {
			this.jobItems[i].currentPage = currentPage;
		}
		if (totalPages !== "" && totalPages !== 0 && totalPages !== -1) {
			this.jobItems[i].totalPages = totalPages;
		}
		if (description !== "") {
			this.jobItems[i].description = description;
		}
		if (typeof blockedReasons !== "undefined" && typeof blockedReasons.length !== "undefined" &&
		    blockedReasons.length > 0) {
			this.jobItems[i].blockedReasons = blockedReasons;
		}
	},
	addNewJobItem: function(jobID, printerState, currentPage, totalPages, description, blockedReasons) {
		var newJobItem = {jobID: jobID, printerState: printerState, currentPage: currentPage,
		                  totalPages: totalPages, description: description, blockedReasons: blockedReasons};
		console.log("PrintMonitorQueueStatus addNewJobItem: newJobItem = " + JSON.stringify(newJobItem));

		if (printerState !== "BLOCKED" && printerState !== "Cancelled" && printerState !== "Failed") {
			if (printerState !== "RUNNING" && printerState !== "Preparing") {
				if (printerState === "DONE") {
					console.log("PrintMonitorQueueStatus addNewJobItem: unexpected done state - remember id so we don't mistakenly show a dashboard for it");
					var idsShownLength = this.idsShown.length;
					this.idsShown[idsShownLength] = jobID;
				}
				else {
					console.log("PrintMonitorQueueStatus addNewJobItem: need Running, Blocked, Preparing, Cancelled, or Failed status to start dashboard");
				}
				return;
			}
			else if (this.idsShown.indexOf(jobID) !== -1) {		// already showed dashboard for this job
				console.log("PrintMonitorQueueStatus addNewJobItem: already created dashboard, aborting");
				return;
			}
		}

		// remember we showed this job before
		if (this.idsShown.indexOf(jobID) === -1) {
			var idsShownLength = this.idsShown.length;
			this.idsShown[idsShownLength] = jobID;
		}

		// add new job
		var jobItemsLength = this.jobItems.length;
		for (var i = jobItemsLength; i >= 1; i--) {
			this.jobItems[i] = this.jobItems[i - 1];
		}
		this.jobItems[0] = newJobItem;
	},
	removeJobItem: function(index) {
		var jobItemsLength = this.jobItems.length;
		for (var i = index; i < (jobItemsLength - 1); i++) {
			this.jobItems[i] = this.jobItems[i + 1];
		}
		this.jobItems.splice(jobItemsLength - 1, 1);
	},
	dashboardSwiped: function() {
		var jobItemsLength = this.jobItems.length;
		if (jobItemsLength >= 1) {
			this.removeJobItem(jobItemsLength - 1);
		}
		else {
			console.log("PrintMonitorQueueStatus dashboardSwiped: trying to remove item from empty job list");
		}
	},
	getLayers: function() {
		var layers = [];

		for (var i = 0; i < this.jobItems.length; i++) {
			console.log("PrintMonitorQueueStatus getLayers: i = " + i);
			var title = "";
			var text = "";
			var iconFile = "images/notif48/normal.png";

			if (this.jobItems[i].printerState === "RUNNING"   || this.jobItems[i].printerState === "BLOCKED" ||
			    this.jobItems[i].printerState === "Cancelled" || this.jobItems[i].printerState === "Failed") {
				var pageStatusString = "";
				if (this.jobItems[i].currentPage !== -1 && this.jobItems[i].currentPage !== "") {
					var descTemplate;
					if (this.jobItems[i].totalPages !== -1 && this.jobItems[i].totalPages !== 0 &&
					    this.jobItems[i].totalPages !== "") {
						console.log("PrintMonitorQueueStatus getLayers: totalPages = " + this.jobItems[i].totalPages);
						descTemplate = new enyo.g11n.Template($L("Page #{pageNumber} of #{totalPages} from #{originName}"));
						pageStatusString = descTemplate.evaluate({pageNumber: this.jobItems[i].currentPage,
						                                          totalPages: this.jobItems[i].totalPages,
						                                          originName: this.jobItems[i].description
							});
					}
					else {
						descTemplate = new enyo.g11n.Template($L("Page #{pageNumber} from #{originName}"));
						pageStatusString = descTemplate.evaluate({pageNumber: this.jobItems[i].currentPage,
						                                          originName: this.jobItems[i].description
							});
					}
				}
				console.log("PrintMonitorQueueStatus getLayers: pageStatusString = " + pageStatusString);

				if (this.jobItems[i].printerState === "RUNNING") {
					title = pageStatusString;
				}
				else if (this.jobItems[i].printerState === "BLOCKED" || this.jobItems[i].printerState === "Cancelled" ||
				         this.jobItems[i].printerState === "Failed") {
					var errorInfo = this.getErrorStringInfo(i);
					console.log("PrintMonitorQueueStatus getLayers: errorInfo = " + JSON.stringify(errorInfo));
					if (typeof errorInfo.title !== "undefined") {
						title = errorInfo.title;
					}

					if (typeof errorInfo.type !== "undefined" && errorInfo.type === "notification") {
						text = pageStatusString;
						iconFile = "images/notif48/warning.png";
					}
					else {
						iconFile = "images/notif48/error.png";
					}
				}
			}
			else if (this.jobItems[i].printerState === "Preparing") {
				title = $L("Preparing Print Job...");
			}

			var jobID = this.jobItems[i].jobID;

			console.log("PrintMonitorQueueStatus getLayers: title = " + title);
			console.log("PrintMonitorQueueStatus getLayers: text = " + text);
			layers[i] = {icon: iconFile, title: title, text: text, jobID: jobID};
		}

		return layers;
	},
	getErrorStringInfo: function(index) {
		var displayStringInfo = [];
		displayStringInfo.title = "";
		displayStringInfo.type = "";

		var errorFlags = [];
		errorFlags.unableToConnect = false;
		errorFlags.busy = false;
		errorFlags.cancelled = false;
		errorFlags.failed = false;
		errorFlags.outOfPaper = false;
		errorFlags.outOfInk = false;
		errorFlags.outOfToner = false;
		errorFlags.jammed = false;
		errorFlags.doorOpen = false;
		errorFlags.serviceRequest = false;
		errorFlags.lowOnInk = false;
		errorFlags.lowOnToner = false;

		// find all errors currently present
		for (var i = 0; i < this.jobItems[index].blockedReasons.length; i++) {
			if (this.jobItems[index].blockedReasons[i] === "Unable_To_Connect") {
				errorFlags.unableToConnect = true;
			}
			else if (this.jobItems[index].blockedReasons[i] === "Busy") {
				errorFlags.busy = true;
			}
			else if (this.jobItems[index].blockedReasons[i] === "Cancelled") {
				errorFlags.cancelled = true;
			}
			else if (this.jobItems[index].blockedReasons[i] === "Failed") {
				errorFlags.failed = true;
			}
			else if (this.jobItems[index].blockedReasons[i] === "Out_Of_Paper") {
				errorFlags.outOfPaper = true;
			}
			else if (this.jobItems[index].blockedReasons[i] === "Out_Of_Ink") {
				errorFlags.outOfInk = true;
			}
			else if (this.jobItems[index].blockedReasons[i] === "Out_Of_Toner") {
				errorFlags.outOfToner = true;
			}
			else if (this.jobItems[index].blockedReasons[i] === "Jammed") {
				errorFlags.jammed = true;
			}
			else if (this.jobItems[index].blockedReasons[i] === "Door_Open") {
				errorFlags.doorOpen = true;
			}
			else if (this.jobItems[index].blockedReasons[i] === "Service_Request") {
				errorFlags.serviceRequest = true;
			}
			else if (this.jobItems[index].blockedReasons[i] === "Low_On_Ink") {
				errorFlags.lowOnInk = true;
			}
			else if (this.jobItems[index].blockedReasons[i] === "Low_On_Toner") {
				errorFlags.lowOnToner = true;
			}
		}

		// look through error flags in priority order to display most important error message
		if (errorFlags.doorOpen === true) {
			displayStringInfo.title = $L("Door Open");
			displayStringInfo.type = "alert";
		}
		else if (errorFlags.outOfPaper === true) {
			displayStringInfo.title = $L("Out of Paper");
			displayStringInfo.type = "alert";
		}
		else if (errorFlags.jammed === true) {
			displayStringInfo.title = $L("Printer Jam");
			displayStringInfo.type = "alert";
		}
		else if (errorFlags.serviceRequest === true) {
			displayStringInfo.title = $L("Printer Error");
			displayStringInfo.type = "alert";
		}
		else if (errorFlags.outOfInk === true) {
			displayStringInfo.title = $L("Out of Ink");
			displayStringInfo.type = "alert";
		}
		else if (errorFlags.outOfToner === true) {
			displayStringInfo.title = $L("Out of Toner");
			displayStringInfo.type = "alert";
		}
		else if (errorFlags.unableToConnect === true) {
			displayStringInfo.title = $L("Unable To Connect");
			displayStringInfo.type = "alert";
		}
		else if (errorFlags.cancelled === true) {
			displayStringInfo.title = $L("Print Job Canceled");
			displayStringInfo.type = "alert";
		}
		else if (errorFlags.failed === true) {
			displayStringInfo.title = $L("Print Job Failed");
			displayStringInfo.type = "alert";
		}
		else if (errorFlags.busy === true) {
			displayStringInfo.title = $L("Printer Busy");
			displayStringInfo.type = "notification";
		}
		else if (errorFlags.lowOnInk === true) {
			displayStringInfo.title = $L("Low on Ink");
			displayStringInfo.type = "notification";
		}
		else if (errorFlags.lowOnToner === true) {
			displayStringInfo.title = $L("Low on Toner");
			displayStringInfo.type = "notification";
		}

		// use general service request message as fallback if reasons list is empty or no valid reason found
		else {
			displayStringInfo.title = $L("Printer Error");
			displayStringInfo.type = "alert";
		}

		return displayStringInfo;
	}
});
