enyo.kind({
	name: "enyo.Canon.PrintPopup",
	kind: enyo.VFlexBox,
	className: "PrintPopup",
	jobID: -1,
	currentStatus: "",
	blockedReasons: [],
	components: [
		{kind: "PrintManagerService", components: [
			{name: "cancelJob", method: "jobs/cancel", onSuccess: "cancelJobSuccess", onFailure: "cancelJobFailure"}
		]},
		{kind: enyo.HFlexBox, tapHighlight: false, flex: 1, components: [
			{kind: "Image", className: "notification-icon", src: "images/notif48/error.png"},
			{kind: enyo.VFlexBox, className:"notification-text-box", components: [
				{content: "", name: "popupTitle", className: "notification-subject"},
				{content: "", name: "popupText", className: "notification-body-multi"}
			]}
		]},
		{kind: enyo.HFlexBox, tapHighlight: false, components: [
			{kind: "NotificationButton", name: "cancelButton", className: "enyo-notification-button-negative", caption: $L("Cancel Print Job"), flex: 1, showing: true, onclick: "cancelClick"},
			{kind: "NotificationButton", caption: $L("Dismiss"), className: "enyo-notification-button-affirmative", flex:1, style:"margin-right:15px", onclick: "dismissClick"}
		]},
		{kind: "ApplicationEvents", onWindowParamsChange: "windowParamsChange"}
	],
	create: function() {
		this.inherited(arguments);
		this.jobID = -1;		// initializers needed since calling superclass create could cause
		this.currentStatus = "";	// these values to be changed
		this.blockedReasons = [];
		console.log("PrintPopup create arguments = " + JSON.stringify(arguments));
		if (typeof arguments !== "undefined" && typeof arguments[0] !== "undefined") {
			if (typeof arguments[0].jobID !== "undefined") {
				this.jobID = arguments[0].jobID;
			}
			if (typeof arguments[0].printerState !== "undefined") {
				this.currentStatus = arguments[0].printerState;
			}
			if (typeof arguments[0].blockedReasons !== "undefined") {
				this.blockedReasons = arguments[0].blockedReasons;
			}
		}
		console.log("PrintPopup create: jobID = " + this.jobID);
		console.log("PrintPopup create: currentStatus = " + this.currentStatus);
		console.log("PrintPopup create: blockedReasons = " + JSON.stringify(this.blockedReasons));

		this.updateStatus();
	},
	windowParamsChange: function() {
		console.log("PrintPopup windowParamsChange: windowParams = " + JSON.stringify(enyo.windowParams));
		if (typeof enyo.windowParams.printerState !== "undefined") {
			this.currentStatus = enyo.windowParams.printerState;
		}
		if (typeof enyo.windowParams.blockedReasons !== "undefined") {
			this.blockedReasons = enyo.windowParams.blockedReasons;
		}

		console.log("PrintPopup windowParamsChange: currentStatus = " + this.currentStatus);
		console.log("PrintPopup windowParamsChange: blockedReasons = " + JSON.stringify(this.blockedReasons));

		if (this.currentStatus !== "BLOCKED") {
			close();
		}
		else {
			this.updateStatus();
		}
	},
	updateStatus: function() {
		var errorFlags = [];
		errorFlags["unableToConnect"] = false;
		errorFlags["busy"] = false;
		errorFlags["cancelled"] = false;
		errorFlags["failed"] = false;
		errorFlags["outOfPaper"] = false;
		errorFlags["outOfInk"] = false;
		errorFlags["outOfToner"] = false;
		errorFlags["jammed"] = false;
		errorFlags["doorOpen"] = false;
		errorFlags["serviceRequest"] = false;
		errorFlags["lowOnInk"] = false;
		errorFlags["lowOnToner"] = false;

		// find all errors currently present
		for (var i = 0; i < this.blockedReasons.length; i++) {
			if (this.blockedReasons[i] === "Unable_To_Connect") {
				errorFlags["unableToConnect"] = true;
			}
			else if (this.blockedReasons[i] === "Busy") {
				errorFlags["busy"] = true;
			}
			else if (this.blockedReasons[i] === "Cancelled") {
				errorFlags["cancelled"] = true;
			}
			else if (this.blockedReasons[i] === "Failed") {
				errorFlags["failed"] = true;
			}
			else if (this.blockedReasons[i] === "Out_Of_Paper") {
				errorFlags["outOfPaper"] = true;
			}
			else if (this.blockedReasons[i] === "Out_Of_Ink") {
				errorFlags["outOfInk"] = true;
			}
			else if (this.blockedReasons[i] === "Out_Of_Toner") {
				errorFlags["outOfToner"] = true;
			}
			else if (this.blockedReasons[i] === "Jammed") {
				errorFlags["jammed"] = true;
			}
			else if (this.blockedReasons[i] === "Door_Open") {
				errorFlags["doorOpen"] = true;
			}
			else if (this.blockedReasons[i] === "Service_Request") {
				errorFlags["serviceRequest"] = true;
			}
			else if (this.blockedReasons[i] === "Low_On_Ink") {
				errorFlags["lowOnInk"] = true;
			}
			else if (this.blockedReasons[i] === "Low_On_Toner") {
				errorFlags["lowOnToner"] = true;
			}
		}

		// look through error flags in priority order to display most important error message
		if (errorFlags["doorOpen"] === true) {
			this.$.popupTitle.setContent($L("Door Open"));
			this.$.popupText.setContent($L("To finish the print job close the printer door."));
			this.$.cancelButton.setShowing(true);
		}
		else if (errorFlags["outOfPaper"] === true) {
			this.$.popupTitle.setContent($L("Out of Paper"));
			this.$.popupText.setContent($L("To finish the print job load more paper in the tray."));
			this.$.cancelButton.setShowing(true);
		}
		else if (errorFlags["jammed"] === true) {
			this.$.popupTitle.setContent($L("Printer Jam"));
			this.$.popupText.setContent($L("To finish the print job clear the paper or carriage jam."));
			this.$.cancelButton.setShowing(true);
		}
		else if (errorFlags["serviceRequest"] === true) {
			this.$.popupTitle.setContent($L("Printer Error"));
			this.$.popupText.setContent($L("Refer to the printer for more information."));
			this.$.cancelButton.setShowing(true);
		}
		else if (errorFlags["outOfInk"] === true) {
			this.$.popupTitle.setContent($L("Out of Ink"));
			this.$.popupText.setContent($L("To finish the print job insert new ink cartridge(s)."));
			this.$.cancelButton.setShowing(true);
		}
		else if (errorFlags["outOfToner"] === true) {
			this.$.popupTitle.setContent($L("Out of Toner"));
			this.$.popupText.setContent($L("To finish the print job insert new toner cartridge(s)."));
			this.$.cancelButton.setShowing(true);
		}
		else if (errorFlags["unableToConnect"] === true) {
			this.$.popupTitle.setContent($L("Unable To Connect"));
			this.$.popupText.setContent($L("A connection with the printer cannot be made."));
			this.$.cancelButton.setShowing(true);
		}
		else if (errorFlags["cancelled"] === true) {
			this.$.popupTitle.setContent($L("Print Job Canceled"));
			this.$.popupText.setContent($L("The print job has been canceled."));
			this.$.cancelButton.setShowing(false);
		}
		else if (errorFlags["failed"] === true) {
			this.$.popupTitle.setContent($L("Print Job Failed"));
			this.$.popupText.setContent($L("The print job has failed.  Try printing again."));
			this.$.cancelButton.setShowing(false);
		}

		// use general service request message as fallback if reasons list is empty or no valid reason found
		else {
			this.$.popupTitle.setContent($L("Printer Error"));
			this.$.popupText.setContent($L("Refer to the printer for more information."));
			this.$.cancelButton.setShowing(true);
		}

		/////  These are warnings and shouldn't be shown in an alert popup
		//
		//else if (errorFlags["busy"] === true) {
		//	this.$.popupTitle.setContent($L("Printer Busy"));
		//	this.$.popupText.setContent($L("The printer is currently busy."));
		//}
		//else if (errorFlags["lowOnInk"] === true) {
		//	this.$.popupTitle.setContent($L("Low on Ink"));
		//}
		//else if (errorFlags["lowOnToner"] === true) {
		//	this.$.popupTitle.setContent($L("Low on Toner"));
		//}
	},
	dismissClick: function() {
		close();
	},
	cancelClick: function() {
		if (this.jobID !== -1) {
			this.$.cancelButton.setDisabled(true);
			this.$.cancelJob.call({jobID: this.jobID});
		}
	},
	cancelJobSuccess: function(inSender, inResponse, inRequest) {
		console.log("PrintPopup cancelJobSuccess: " + JSON.stringify(inResponse));
	},
	cancelJobFailure: function(inSender, inResponse, inRequest) {
		console.log("PrintPopup cancelJobFailure: " + JSON.stringify(inResponse));
		console.log(inSender.getName() + " FAIL: " + inResponse.errorText);
		this.$.cancelButton.setDisabled(false);
	}
});